@@ -9,10 +9,13 @@ use clap::Parser;
 #[derive(Parser)]
 #[clap(author, version, about)]
 pub struct ToipeConfig {
-    /// word list name or path to word list file.
+    /// Word list name or path to word list file.
     /// Available word lists:
     /// top250,
     /// os
     #[clap(short, long, default_value_t = String::from("top250"))]
     pub wordlist: String,
+    /// Number of words to show on each test.
+    #[clap(short, long, default_value_t = 30)]
+    pub num_words: usize,
 }
