@@ -1,5 +1,8 @@
+//! Utilities for generating/selecting new (random) words for the typing
+//! test.
+
 use std::fs::File;
-use std::io::{self, BufRead, BufReader, Seek, SeekFrom};
+use std::io::{self, BufRead, BufReader, Cursor, Seek, SeekFrom};
 use std::path::PathBuf;
 
 use rand::Rng;
@@ -9,25 +12,63 @@ extern crate bisection;
 use bisection::bisect_right;
 use rand::prelude::ThreadRng;
 
+/// Efficient selector of words from a word list.
+///
+/// The word list is given by a BufReader.
+///
+/// ### Assumptions
+///
+/// The word list is assumed to:
+/// - Have a list of words separated by newline.
+/// - Use only English alphabet and **ASCII**.
+/// - Be **sorted alphabetically**.
+///     - In case-insensitive manner.
+///     - For example, both "Apple" and "apple" must appear before words
+///       started with "b".
+/// - Be a file that is **not modified** while the object is alive.
+/// - Have no empty lines except at the end of the file.
+///
+/// Note: only words between length 2 and 8, inclusive, are considered.
+/// Having no words matching the criteria may lead to an infinite loop.
+///
+/// ### Algorithm
+///
+/// During initialization, the [`RawWordSelector`] iterates through all
+/// the words in the list and builds an index mapping each letter (of
+/// the alphabet) to its byte position in the file and the cumulative
+/// number of words present starting with it.
+///
+/// To select a (pesudo-)random word, a random number between 0
+/// (inclusive) and number of lines (exclusive) is generated. Using
+/// binary search, the index of where this number lies in the cumulative
+/// no. of words list is found. Using this index, the byte offset of the
+/// corresponding letter is found. The file is then read starting from
+/// this byte offset and read line-by-line until the correct word (at
+/// line `number - cumulative num. words` from the starting of this
+/// letter).
+///
+/// ### Time complexity
+///
+/// Initialization: `O(n)`
+///
+/// Selecting a word: `O(1)` (best case) or `O(n)` (worst case)
+///
+/// ### Space complexity
+///
+/// `O(1)` (only needs fixed length arrays).
 #[derive(Debug)]
-pub struct WordSelector {
-    word_list_path: PathBuf,
+pub struct RawWordSelector<T> {
+    reader: BufReader<T>,
     letter_pos: [u64; 26],
     letter_lines_sum: [u64; 27],
 }
 
-impl Default for WordSelector {
-    fn default() -> Self {
-        Self::new(PathBuf::from("/usr/share/dict/words")).unwrap()
-    }
-}
-
-impl WordSelector {
-    pub fn new(word_list_path: PathBuf) -> Result<Self, io::Error> {
-        let file = File::open(word_list_path.clone())?;
-
-        let mut reader = BufReader::new(file);
-
+impl<T: Seek + io::Read> RawWordSelector<T> {
+    /// Create from any arbitrary [`BufReader`].
+    ///
+    /// Please ensure that assumptions defined at
+    /// [`RawWordSelector#assumptions`] are valid for the contents.
+    pub fn new(mut reader: BufReader<T>) -> Result<Self, io::Error> {
         let mut letter_pos = [0u64; 26];
         let mut letter_lines = [0u64; 26];
         let mut num_lines = 0;
@@ -98,7 +139,7 @@ impl WordSelector {
         // println!("{:?}", letter_lines_sum);
 
         let word_selector = Self {
-            word_list_path,
+            reader,
             letter_pos,
             letter_lines_sum,
         };
@@ -107,22 +148,19 @@ impl WordSelector {
     }
 
     fn word_at_letter_offset(
-        &self,
+        &mut self,
         letter_index: usize,
         line_offset: u64,
     ) -> Result<String, io::Error> {
-        let file = File::open(self.word_list_path.clone())?;
-
-        let mut reader = BufReader::new(file);
-
-        reader.seek(SeekFrom::Start(self.letter_pos[letter_index]))?;
+        self.reader
+            .seek(SeekFrom::Start(self.letter_pos[letter_index]))?;
 
         let mut buffer = String::new();
         let mut line_no = 0;
 
         loop {
             buffer.clear();
-            reader.read_line(&mut buffer)?;
+            self.reader.read_line(&mut buffer)?;
 
             if line_no == line_offset {
                 break;
@@ -137,7 +175,7 @@ impl WordSelector {
         Ok(buffer)
     }
 
-    fn new_word_raw(&self, rng: &mut ThreadRng) -> Result<String, io::Error> {
+    fn new_word_raw(&mut self, rng: &mut ThreadRng) -> Result<String, io::Error> {
         let line_index = rng.gen_range(self.letter_lines_sum[0]..self.letter_lines_sum[26]);
         // let line_index = 0;
 
@@ -157,8 +195,51 @@ impl WordSelector {
 
         self.word_at_letter_offset(letter_index, line_offset)
     }
+}
 
-    pub fn new_word(&self) -> Result<String, io::Error> {
+impl RawWordSelector<File> {
+    /// Create from a file at a path given by a [`PathBuf`].
+    ///
+    /// Please ensure that assumptions defined at
+    /// [`RawWordSelector#assumptions`] are valid for this file.
+    pub fn from_path(word_list_path: PathBuf) -> Result<Self, io::Error> {
+        let file = File::open(word_list_path.clone())?;
+
+        let reader = BufReader::new(file);
+
+        Ok(Self::new(reader)?)
+    }
+}
+
+impl RawWordSelector<Cursor<String>> {
+    /// Create from a String representing the word list file.
+    ///
+    /// Please ensure that assumptions defined at
+    /// [`RawWordSelector#assumptions`] are valid for the contents.
+    pub fn from_string(word_list: String) -> Result<Self, io::Error> {
+        let cursor = Cursor::new(word_list.clone());
+        let reader = BufReader::new(cursor);
+
+        Ok(RawWordSelector::new(reader)?)
+    }
+}
+
+/// Describes a thing that provides new words.
+pub trait WordSelector {
+    /// Returns a new word.
+    fn new_word(&mut self) -> Result<String, io::Error>;
+
+    /// Returns a [`Vec`] containing `num_words` words.
+    fn new_words(&mut self, num_words: usize) -> Result<Vec<String>, io::Error> {
+        (0..num_words)
+            .into_iter()
+            .map(|_| self.new_word())
+            .collect()
+    }
+}
+
+impl<T: Seek + io::Read> WordSelector for RawWordSelector<T> {
+    fn new_word(&mut self) -> Result<String, io::Error> {
         let mut rng = rand::thread_rng();
 
         let mut word = "-".to_string();
@@ -171,11 +252,4 @@ impl WordSelector {
 
         Ok(word)
     }
-
-    pub fn new_words(&self, num_words: usize) -> Result<Vec<String>, io::Error> {
-        (0..num_words)
-            .into_iter()
-            .map(|_| self.new_word())
-            .collect()
-    }
 }
